
# q-expansion of newform 495.4.a.d, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 495
weight = 4
poly_data = [-4, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 1], [0, 0], [5, 0], [-4, 4], [11, 0], [-44, -2], [30, -44], [-74, -22], [32, 60], [96, -34], [36, -12], [-130, -112], [-96, 154], [-196, -124], [-4, -216], [-334, 196], [-4, -240], [-146, 364], [-380, 16], [-1008, -44], [-272, 58], [474, -306], [-70, 426], [-186, 128], [-298, 428], [-308, 26], [-512, 656], [1282, 322], [-746, 484], [-170, 0], [-1440, 192], [368, 436], [186, -340], [-230, 414], [1936, -282], [-46, 30], [-1038, 540], [-804, 1064], [714, -346], [1138, -44], [268, -752], [1882, -1524], [-16, -556], [-2588, -82], [-2334, 732], [1584, -1544], [2286, 794], [-1768, -808], [4558, 174], [3442, -1452], [-2690, 164], [1904, -112], [3290, 920], [3820, 936], [-6350, -532], [-1482, 498], [-5022, 1292], [3022, -1366], [7260, -1162], [800, -1670], [-7972, 548], [-350, -3124], [2392, 1168], [4936, -1228], [-1014, 80], [-6062, -1828], [-4388, 3040], [-6640, -2030], [842, 482], [2118, 1452], [-3970, 2496], [-1620, 2148], [-6096, 4592], [-2812, 1166], [4136, -1516], [-3740, 5760], [-10538, 952], [-3406, 1164], [-3758, 3772], [-730, 820], [-3236, 768], [4818, -6660], [444, -3316], [-15162, -548], [4506, 78], [552, -6108], [-6322, 2088], [6236, -2410], [11400, 182], [-5952, -2808], [-4004, 5536], [5360, -3432], [3376, -2424], [-8040, 4180], [-7100, -1576], [4258, -5090], [-11374, -868], [-5958, -3140], [-15672, 360], [982, 196], [1840, -7184], [7510, 512], [15202, -3598], [-712, 1626], [-11018, -4086], [3950, -6732], [11604, -7144], [13170, -1144], [7712, 6316], [12950, -9484], [-16688, -3648], [4884, -3690], [358, 4840], [7140, -3232], [7608, 152], [15758, -5928], [-19532, 1368], [18156, -6064], [7718, -4224], [25492, -1816], [-13090, 12232], [-8684, -1586], [-15002, 8856], [6120, 5964], [14672, 4476], [-21564, 3194], [-3250, 8360], [-1192, -3308], [-17976, -4608], [7544, 10730], [15710, 2962], [8866, -11882], [3800, 9664], [5282, 988], [8616, -4486], [14526, -16468], [-12690, 6488], [-26300, 2020], [12618, -9492], [-2276, -4698], [27154, -2130], [-24920, 8318], [12880, -6888], [39202, -2198], [-21242, 15312], [-3704, 17116], [620, -6242], [-11954, 636], [-22932, -1632], [-17164, -5192], [-3708, -538], [11394, -300], [25756, -4936], [3966, 4090], [-40740, -1656], [19624, -10332], [9002, -5370], [-1478, 3508], [-6752, 5970], [-3020, -2374], [-39968, -2956], [-5894, -16192], [4096, -15720], [24172, 2944], [-39758, 6620], [7916, 14320], [4916, -15500], [-13388, -2722], [13738, 10152], [12930, -10196], [42396, -12400], [-25778, 11264], [-38428, 14748], [-10420, 16814], [-4056, 8616], [3930, 6932], [-19602, 25826], [23258, -3404], [52408, -5896], [-24502, -14760], [41056, -15528], [6012, 12424], [-30014, 3068], [-8950, 13424], [1412, -2328], [20770, -21140], [-10752, -12294], [13004, 15776], [5590, -15868], [-17308, -3004], [58190, -8444], [-9670, 4882], [22868, -24424], [-24746, 26064], [-6686, 9090], [-37254, 3004], [-19426, 27460], [58590, -2352], [-13094, -12824], [-8150, -10906], [-3052, -15046], [-58330, 10458], [16134, -7728], [-4670, 10536], [-12484, 9208], [36746, 4420], [26188, 17148], [2834, 4298], [-6148, 12118], [32184, 9300], [-7208, 10234], [82354, -3060], [4888, -12600], [13884, -2760], [62716, 1956], [-49738, -13388], [-8616, -25536], [49276, -26186], [-20272, -9884], [-34634, -13264], [-29182, 4040], [-27630, 28174], [14914, -10564], [14320, 9520], [76718, -2970], [9314, -38152], [28170, 5748], [-49672, 12836], [-39764, -25692], [46628, -23624], [7154, -11100], [-75542, 3438], [76014, -13534], [-32404, -14394], [-63212, 8896], [-12126, 38230], [24510, -40652], [-23126, 20116], [83628, -12328], [-80496, 14820], [-34672, -21636], [45490, -42602], [23184, -23328], [21274, -14540], [-37042, -6196], [51492, -2916], [-3224, 25720], [54724, -24464], [1726, 39618], [36690, 16814], [5428, 13414], [-30612, -11994], [-36912, -12052], [9238, -13324], [-21834, -672], [-3816, 34404], [37198, -4112], [-61668, 41804], [74, -13044], [-50144, 15018], [31832, -7128], [-27662, 25658], [-11086, -17440], [45792, -10374], [-61218, 34248], [-38196, 24856], [-5110, -28948], [-80430, -18916], [35408, -45856], [42718, -51968], [11962, 35364], [-22468, -5184], [-30274, 20156], [58506, -19450], [13324, -3082], [62880, 26120], [39672, -8652], [-70922, 32500], [29814, -19084], [-82200, 19796], [-29210, 20938], [48596, 19604], [-85934, 10086], [-64798, -26280], [11320, -10976], [-32376, 30908], [-64834, 28980], [112838, 10024], [-48412, 8020], [-18924, 64174], [90294, 22032], [93672, -1908], [-31866, 39228], [-25688, -43556], [-88752, 27130], [4704, -30354], [46520, -6648], [40742, -46528], [35120, 15684], [-79328, 6592], [-46088, 37634], [49254, -6992], [49054, -30670], [-33548, 5608], [-72816, -14148], [103322, 1128], [5056, -25348], [104494, -45608], [-40696, -40308], [-44876, -5938], [12402, -23226], [-45682, 65928], [-3396, -42394], [-15676, 73368], [3322, 29742], [17490, 4096], [63916, -25136], [-11108, 2372], [22438, -66532], [-46184, 42402], [29646, -17886], [162214, 3156], [108420, -46246], [-102248, 37464], [-87954, -25308], [-16586, -11316], [-22648, -12260], [-2140, -70016], [-15314, -40406], [53098, -15356], [-63038, 32976], [-41946, 81072], [20346, 53958], [50094, 8846], [-4870, 68110], [62552, 17644], [172138, 6260], [-85242, 56388], [104110, 15004], [53292, 564], [-127806, 32932], [-49978, 39176], [-64240, -38474], [-34784, 60216], [105062, 5048], [11028, -64968], [63422, -66232], [12452, -63352], [51112, -30768], [-44602, -67280], [80706, -65722], [11602, 33560], [165210, -37580], [30720, 59168], [-190250, -5348], [79670, -6908], [69392, -18468], [78524, -26144], [37194, -67544], [83824, 21628], [194896, -10310], [-94684, 21798], [16240, -51996], [10692, -21864], [23428, -16512], [-150394, -13052], [-85474, 12728], [45868, 46524], [-159354, -10996], [-110884, 53400], [-164492, 12680], [66450, -2986], [70460, -7014], [-169854, 45670], [-47806, 77780], [88660, -17296], [69336, 8712], [-98936, -22078], [20672, -39014], [87460, 2958], [97790, -38824], [119370, 25860], [127720, -9680], [-54314, 19100], [91354, 38182], [-38640, -40756], [-76776, -38864], [-302, 37676], [-55056, 26192], [174016, -3068], [-175030, 14408], [-64558, 63852], [-58156, -30352], [-66748, -73864], [148216, 34044], [43172, -17890], [137470, -12542], [-22722, 15168], [95720, -68204], [-124640, 38742], [172130, -904], [-140170, 31464], [-31572, -47652], [-74662, 20772], [-90304, -64442], [-60614, 27134], [-69366, 20076], [-55564, 4734], [-88900, 63160], [156140, 27544], [-131736, -15534], [-185926, 58596], [107700, -82280], [-163282, -35526], [29008, -13670], [-59690, -59496], [-202308, 40644], [-40672, -8496], [22030, -67208], [164218, 5606], [-5070, 53252], [-44144, 84842], [-9424, 84108], [-156540, 50344], [-90078, 11112], [225774, -8352], [64196, -96800], [-172900, -28402], [219872, 32060], [73936, 45888]]
